import atexit
import threading


class BaseThreadWorker:
    def __init__(self, log_queue, formatter):
        self.log_queue = log_queue
        self.worker_thread = None
        self.record = None
        self.formatter = formatter
        atexit.register(self.end)

    def output(self):
        raise NotImplementedError('output must be implemented by ThreadWorker subclasses')

    def worker(self):
        while True:
            self.record = self.log_queue.get()
            if self.record is None:
                break
            self.output()

    def start(self):
        self.worker_thread = threading.Thread(target=self.worker)
        self.worker_thread.daemon = True
        self.worker_thread.start()

    def end(self):
        self.log_queue.put(None)
        self.worker_thread.join()

    def format(self, record):
        return self.formatter.format(record)



import logging
import queue


class QueueHandler(logging.Handler):
    def __init__(self):
        super().__init__()
        self.log_queue = queue.Queue(-1)

    def emit(self, record):
        self.log_queue.put(record)



import logging
import os
import shutil

from q_logger_py.worker.base_thread_worker import BaseThreadWorker


class ThreadRotateFileWorker(BaseThreadWorker):
    def __init__(self, log_queue, filename, max_bytes=50000, backup_count=5, formatter=logging.Formatter()):
        super().__init__(log_queue, formatter)
        self.filename = filename
        self.max_bytes = max_bytes
        self.backup_count = backup_count
        self.file = open(self.filename, 'a')

    def output(self):
        log_output = self.format(self.record)
        self.file.write(log_output + '\n')
        self.file.flush()

        if self.file.tell() > self.max_bytes:
            self.rotate()

    def rotate(self):
        self.file.close()
        for i in range(self.backup_count - 1, 0, -1):
            src = f"{self.filename}.{i}"
            dst = f"{self.filename}.{i + 1}"
            if os.path.exists(src):
                shutil.move(src, dst)
        shutil.move(self.filename, f"{self.filename}.1")
        self.file = open(self.filename, 'a')

    def __del__(self):
        self.file.close()



import logging
import sys

from q_logger_py.worker.base_thread_worker import BaseThreadWorker


class ThreadStdoutWorker(BaseThreadWorker):
    def __init__(self, log_queue, formatter=logging.Formatter()):
        super().__init__(log_queue, formatter)

    def output(self):
        log_output = self.format(self.record)
        sys.stdout.write(log_output + '\n')



from q_logger_py.handler.queue_handler import QueueHandler
from q_logger_py.worker.thread_file_worker import ThreadRotateFileWorker
from q_logger_py.worker.thread_stdout_worker import ThreadStdoutWorker

__all__ = ["QueueHandler",
           "ThreadRotateFileWorker", "ThreadStdoutWorker"]
